/*
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/bonsai
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
 *               Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
 *               Marian Pohling, Lukas Twardon, Lukas Kettenbach
 */
package de.unibi.airobots.bonsai.core;

import java.io.IOException;

/**
 * Interface for sensors that defines two ways of receiving data from a sensor.
 * One possibility is to use the synchronous interface with {@link #readLast()}.
 * If this is not desired, an asynchronous interface can be used based on the
 * observer pattern.
 * 
 * @author lschilli
 * @author dklotz
 * @author jwienke
 * 
 * @param <T>
 *            data type read by this sensor
 */
public interface Sensor<T> extends ManagedCoreObject {

    /**
     * Returns a class object describing the data returned by this sensor.
     * 
     * @return data type class
     */
    Class<T> getDataType();

    // synchronous api ---------------------------------------------------------

    // TODO think about the exceptions used by this interface

    /**
     * Reads the last (newest) data received by this sensor, blocks if this
     * necessary to provide any data.
     * 
     * @return last received data by this sensor
     * @throws IOException
     *             communication error with the data provider
     * @throws InterruptedException
     *             interrupted while waiting for new data
     */
    T readLast() throws IOException, InterruptedException;

    /**
     * Reads the last (newest) data this sensor can provide and waits a maximum
     * time if there is no data available.
     * 
     * @param timeout
     *            time to wait for new data in ms.
     * @return data or <code>null</code> if timeout was reached
     * @throws IOException
     *             communication error with the data provider
     * @throws InterruptedException
     *             interrupted while waiting for new data
     */
    T readLast(long timeout) throws IOException, InterruptedException;

    /**
     * Tells if there are new data available that can be read via one of the
     * <code>readLast</code> methods.
     * 
     * @return <code>true</code> if new data are available, else
     *         <code>false</code>
     */
    boolean hasNext();

    // asynchronous api --------------------------------------------------------

    /**
     * Adds a new listener to this sensor that will be notified asynchronously
     * the moment new data is available.
     * 
     * @param listener
     *            new listener to add to this sensor
     */
    void addSensorListener(SensorListener<T> listener);

    /**
     * Removes the specified listener from this sensor so that it will not be
     * notified for new data.
     * 
     * @param listener
     *            listener to remove
     */
    void removeSensorListener(SensorListener<T> listener);

}
