/*
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/bonsai
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
 *               Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
 *               Marian Pohling, Lukas Twardon, Lukas Kettenbach
 */
package de.unibi.airobots.bonsai.skills.states;

import de.unibi.airobots.bonsai.actuators.NavigationActuator;
import de.unibi.airobots.bonsai.core.Sensor;
import de.unibi.airobots.bonsai.skills.SkillStateMachine;
import de.unibi.airobots.bonsai.skills.abstractstates.AbstractState;
import de.unibi.airobots.bonsai.skills.abstractstates.exceptions.EndException;
import de.unibi.airobots.bonsai.skills.abstractstates.exceptions.ExecuteException;
import de.unibi.airobots.bonsai.skills.abstractstates.exceptions.InitException;
import de.unibi.airobots.bonsai.strategies.navigation.NavigationStrategy;
import de.unibi.airobots.bonsai.strategies.navigation.SimpleNavigationStrategy;
import de.unibi.airobots.bonsai.util.helper.GoalReachedWaiter.ReachStatus;
import de.unibi.airobots.btl.data.navigation.GlobalPlannerStateChange;
import de.unibi.airobots.btl.data.navigation.NavigationGoalData;
import de.unibi.airobots.btl.data.navigation.NavigationGoalData.GoalType;
import de.unibi.airobots.btl.data.navigation.PositionData;
import de.unibi.airobots.btl.data.navigation.SpeedData;
import de.unibi.airobots.btl.units.AngleUnit;
import de.unibi.airobots.btl.units.LengthUnit;

import java.io.IOException;
import org.apache.log4j.Logger;

/**
 * Use this state to leave the arena. The robot will try to drive to position
 * x=0, y=0 and yaw=0.
 * 
 * @author lkettenb
 */
public class LeaveArena extends AbstractState {
	/*
	 * Sensors used by this state.
	 */
	private Sensor<GlobalPlannerStateChange> gplanChangeSensor;
	private Sensor<PositionData> positionSensor;
	private Sensor<SpeedData> speedSensor;

	/*
	 * Actuators used by this state.
	 */
	private NavigationActuator navActuator;

	/**
	 * The logger.
	 */
	private Logger logger = super.getLogger();

	/**
	 * Constructor.
	 * 
	 * @param aStateMachine
	 *            The state machine to use.
	 */
	public LeaveArena(SkillStateMachine aStateMachine) {
		super(aStateMachine);
	}

	@Override
	public ExitStatus init() throws InitException {
		// Initialize actuators
		navActuator = getManagedActuator("NavigationActuator");

		// Initialize sensors
		positionSensor = getManagedSensor("PositionSensor");
		gplanChangeSensor = getManagedSensor("GlobalPlannerStateChangeSensor");
		speedSensor = getManagedSensor("SpeedSensor");
		return ExitStatus.SUCCESS;
	}

	@Override
	public ExitStatus execute() throws ExecuteException {
		NavigationStrategy navigationStrategy = new SimpleNavigationStrategy(
				navActuator, positionSensor, speedSensor, gplanChangeSensor);

		NavigationGoalData target = new NavigationGoalData(new PositionData(0,
				0, 0, System.currentTimeMillis(), LengthUnit.METER,
				AngleUnit.RADIAN));
		target.setYawTolerance(Math.PI / 5, AngleUnit.RADIAN);
		target.setType(GoalType.GLOBAL);

		ReachStatus reachStatus;

		try {
			reachStatus = navigationStrategy.driveTo(target);
		} catch (IOException ex) {
			// Did i reach my goal? Nobody knows!
			throw new ExecuteException(ex.getMessage());
		}

		if (reachStatus.compareTo(ReachStatus.REACHED) == 0) {
			return ExitStatus.SUCCESS;
		} else {
			logger.error("Error while trying to reach goal.");
			return ExitStatus.ERROR;
		}
	}

	@Override
	public ExitStatus end() throws EndException {
		return ExitStatus.SUCCESS;
	}

}
