package de.unibi.airobots.bonsai.util.arm;

	import java.io.IOException;
	import java.util.ArrayList;

	import org.apache.log4j.Level;
	import org.apache.log4j.Logger;

	import Jama.Matrix;
	import de.unibi.airobots.bonsai.actuators.PoseActuatorTobi;
	import de.unibi.airobots.btl.data.geometry.Point3D;
	import de.unibi.airobots.btl.data.grasp.KatanaPoseData;
	import de.unibi.airobots.btl.data.grasp.PoseData;
	import de.unibi.airobots.btl.units.AngleUnit;
	import de.unibi.airobots.btl.units.LengthUnit;

	/**
	 * ArmControl.java Moves the arm to a desired location observed in the image by
	 * transforming the point using a transformation matrix.
	 * 
	 * @author abaumann, mdulisch
	 */
	public class ArmController180 {

		private static final Logger logger = Logger.getLogger(ArmController.class);
		private static final int UP = 1;
		private static final int DOWN = 0;
		/**
		 * Size of an approach step in millimeters.
		 */
		private static double GRASP_APPROACH_STEP = 20;
		/**
		 * Distance in millimeters between inner gripper and object to grasp before
		 * closing the gripper.
		 */
		private static final double GRASP_MAX_APPROACH_DIST = 30;
		private static final double GRASP_MAX_APPROACH_Z = 500;
		private static final double GRASP_MAX_APPROACH_STEP_COUNT = 8;
		private static final double GRASP_MIN_X = -150;
		private static final double GRASP_MAX_X = 200;
		private static final double GRASP_MAX_Z = 350;
		private static final double GRASP_MAX_Y = 300;
		private static final double GRASP_MIN_Y = -200;
		/**
		 * Default distance in millimeters between gripper (center of calibration
		 * tool!) and object.
		 */
		private static final double GRASP_DEFAULT_DIST = 15;
		private static final Double GRIPPER_ENCODER_CLOSE = 12400.0;
		private PoseActuatorTobi poseActuator;
		private int lastHeight;
		private KatanaPoseData carry;
		private double GRASP_APPROACH_X;
		private double GRASP_APPROACH_Z;
		private CameraArmTransformation cat;
		private boolean tableheight = false;;

		/**
		 * Creates a new ArmControl.
		 */
		public ArmController180(PoseActuatorTobi poseA) {
			this.cat = new CameraArmTransformation();
			// this.setTransformationMatrix("transformationMatrixOutput");
			this.poseActuator = poseA;
		}

		/**
		 * 
		 * moves the arm for wiping.
		 * 
		 * @param wipevalue
		 *            between 0(left) and 1(right).
		 */
		public void wipe(double wipevalue) {
			// if (!tableheight ) {
			// try {
			// this.poseActuator.setMovements("go_down_table");
			// tableheight = true;
			// } catch (IOException e) {
			// e.printStackTrace();
			// }
			// }
			try {
				if ((wipevalue <= 1) && (wipevalue >= 0)) {
					this.poseActuator.moveJoint(0, 5.4 - 1.8 * wipevalue);
					// this.poseActuator.moveJoint(4, 2.4 + 1.6 * wipevalue);
					waitAfterMovement(1000);
				} else {
					logger.debug("wipevalue out of range");
				}
			} catch (IOException e) {
				e.printStackTrace();
			}

			// try {
			// if (direction.toLowerCase().equals("left")) {
			// this.poseActuator.setMovements("swift_left");
			// }
			// if (direction.toLowerCase().equals("right")) {
			// this.poseActuator.setMovements("swift_right");
			// }
			// this.waitAfterMovement();
			//
			// } catch (IOException e) {
			// logger.debug("error movement not in movementfile" + e.toString());
			// }
		}

		/**
		 * wipes to the left site.
		 */
		public void wipeLeft() {
			wipe(0);
		}

		/**
		 * wipes to the right site.
		 */
		public void wipeRight() {
			wipe(1);
		}

		/**
		 * Alters the current arm position by the given differentials.
		 * 
		 * @param dx
		 *            Differential in x direction.
		 * @param dy
		 *            Differential in y direction.
		 * @param dz
		 *            Differential in z direction.
		 */
		public boolean adjustCurrentPosition(double dx, double dy, double dz) {
			KatanaPoseData aktPos;
			try {
				aktPos = this.poseActuator.getPosition();

				Point3D armCoordsPoint = new Point3D(
						aktPos.getX(LengthUnit.MILLIMETER) + dx,
						aktPos.getY(LengthUnit.MILLIMETER) + dy,
						aktPos.getZ(LengthUnit.MILLIMETER) + dz,
						LengthUnit.MILLIMETER);

				return moveTo(armCoordsPoint);
			} catch (IOException e) {
				logger.error("Error adjusting current position.", e);
				return false;
			}
		}

		/**
		 * Helper function to determine the value of GRASP_MAX_APPROACH_DIST. Echoes
		 * the measured distance of the middle infrared sensor continuously. Delete
		 * this, after evaluating a good value or just keep it ;)
		 */
		public void findGraspMaxApproachDist() {
			try {
				this.poseActuator.setMovements("grasp_up_init");
				this.waitAfterMovement();
				while (true) {
					System.out.println(this.poseActuator.getGipperSensorData()
							.getInfraredMiddle());
				}
			} catch (IOException e) {
				logger.error("An error occured while getting infrared data:", e);
			}
		}

		/**
		 * Returns the current position of the arm.
		 * 
		 * @return the current position of the arm.
		 */
		public KatanaPoseData getPosition() {
			try {
				return this.poseActuator.getPosition();
			} catch (IOException e) {
				logger.error("Error getting position.", e);
				e.printStackTrace();
				return null;
			}
		}

		/**
		 * closes the gripper and stops closing when blocked.
		 * 
		 * @return true if an object was in the gripper.
		 */
		public boolean closeGripperByForce() {
			try {
				this.poseActuator.closeGripperByForce();
				if (this.poseActuator.listJoints().get(5) > GRIPPER_ENCODER_CLOSE) {
					// gripper was not closed completely.
					// seems like an object was grasped.
					logger.debug("Seems like an object was grasped. Gripper encoder value: "
							+ this.poseActuator.listJoints().get(5)
							+ " > "
							+ GRIPPER_ENCODER_CLOSE);
					return true;
				} else {
					logger.debug("Seems like no object was grasped. Gripper encoder value: "
							+ this.poseActuator.listJoints().get(5)
							+ " <= "
							+ GRIPPER_ENCODER_CLOSE);
					return false;
				}
			} catch (IOException e) {
				Logger.getRootLogger().error("Error while closing by force");
				e.printStackTrace();
				return false;
			}
		}

		/**
		 * Moves the arm to a safe position depending on the current state.
		 */
		public void goInit() {
			try {
				switch (this.lastHeight) {
				case UP:
					this.poseActuator.setMovements("init_up");
					this.waitAfterMovement();
					break;
				case DOWN:
					this.poseActuator.setMovements("init_down");
					this.waitAfterMovement();
				}
			} catch (IOException e) {
				Logger.getRootLogger().error("Error while going init:");
				e.printStackTrace();
			}
		}

		/**
		 * Moves the arm to the carry position depending on the current state.
		 */
		public void goCarry() {
			tableheight = false;
			try {
				// LinkedList<Double> tempCarryAngles =
				// this.poseActuator.listJoints();
				switch (this.lastHeight) {
				case UP:
					// this.poseActuator.setCarryMovements("up_init");
					// this.goTo(this.carry);
					this.poseActuator.setMovements("go_carry_up");
					this.waitAfterMovement();
					break;
				case DOWN:
					// this.poseActuator.setCarryMovements("down_init");
					// this.goTo(this.carry);
					this.poseActuator.setMovements("go_carry_down");
					this.waitAfterMovement();
				}
			} catch (Exception e) {
				Logger.getRootLogger().error("Error while going carry:");
				e.printStackTrace();
			}
		}
// NEWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW START
		/**
		 * Moves the arm back into its home-position
		 */
		public void goHome() {
			try {
				switch (this.lastHeight) {
				case UP:
					this.poseActuator.setMovements("home_from_grasp_up");
					this.waitAfterMovement();
					break;
				default:
					this.poseActuator.setMovements("home");
					this.waitAfterMovement();
				}
			} catch (IOException e) {
				Logger.getRootLogger().error("Error while going home:");
				e.printStackTrace();
			}
		}
		
		/**
		 * Moves the arm out to start grasping.
		 */
		public void graspUp() {
			try {
				this.poseActuator.setMovements("grasp_up");
				this.lastHeight = UP;
				this.waitAfterMovement();
				this.setCarry();
			} catch (IOException e) {
				Logger.getRootLogger().error("Error while going out.");
				e.printStackTrace();
			}
		}
		
		/**
		 * Lays Arm down with motors off.
		 */
		public void layDown() {
			try {
				this.poseActuator.setMovements("lay_down");
				this.waitAfterMovement();
				this.poseActuator.motorsOff();
			} catch (IOException e) {
				Logger.getRootLogger().error("Error while going out.");
				e.printStackTrace();
			}
		}

// NEWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW END
		
		/**
		 * Moves the arm out to start grasping.
		 */
		public void goOut() {
			try {
				this.poseActuator.setMovements("grasp_down_humble");
				this.waitAfterMovement();
				// this.carryAngles = this.poseActuator.listJoints();
				this.setCarry();
			} catch (IOException e) {
				Logger.getRootLogger().error("Error while going out.");
				e.printStackTrace();
			}
		}

		public void goOutClean() {
			try {
				this.poseActuator.setMovements("go_out_carry_down");
				this.waitAfterMovement();
				// this.carryAngles = this.poseActuator.listJoints();
				this.setCarry();
			} catch (IOException e) {
				Logger.getRootLogger().error("Error while going out.");
				e.printStackTrace();
			}
		}

		/**
		 * Switches the arm from the upper to the lower grasping position and vice
		 * verser.
		 */
		public void twist() {
			try {
				switch (this.lastHeight) {
				case UP:
					this.poseActuator.setMovements("twist_down");
					this.waitAfterMovement();
					this.lastHeight = DOWN;
					break;
				case DOWN:
					this.poseActuator.setMovements("twist_up");
					this.waitAfterMovement();
					this.lastHeight = UP;
				}
			} catch (IOException e) {
				Logger.getRootLogger().error("Error while going home:");
				e.printStackTrace();
			}

		}

		/**
		 * ugly and unnecessary! Keep for writing.
		 * 
		 * @param kpd
		 *            KatanaPoseData to go to.
		 */
		public void goTo(KatanaPoseData kpd) {
			try {
				this.poseActuator.goTo(kpd, true);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		/**
		 * Moves the arm to desired position in image coordinates.
		 * 
		 * @param p
		 *            Point of image coordinates.
		 */
		public boolean goTo(Point3D p) {
			return this.goTo(p, 0);
		}

		/**
		 * Moves the arm to desired position in image coordinates.
		 * 
		 * @param p
		 *            Point of image coordinates.
		 * @param distance
		 *            Distance in millimeters to keep in front of the point.
		 */
		public boolean goTo(Point3D p, double distance) {
			double[][] kinectCoordsSource = { { p.getX(LengthUnit.MILLIMETER) },
					{ p.getY(LengthUnit.MILLIMETER) },
					{ p.getZ(LengthUnit.MILLIMETER) }, { 1.0 } };
			Matrix kinectCoords = new Matrix(kinectCoordsSource);
			Matrix armCoords = this.cat.transform(kinectCoords);
			double armX = armCoords.get(0, 0) / armCoords.get(3, 0);
			double armY = armCoords.get(1, 0) / armCoords.get(3, 0);
			double armZ = armCoords.get(2, 0) / armCoords.get(3, 0);

			Point3D armCoordsPoint = new Point3D(armX, armY, armZ,
					LengthUnit.MILLIMETER);

			armCoordsPoint.setZ(armCoordsPoint.getZ(LengthUnit.MILLIMETER)
					- distance, LengthUnit.MILLIMETER);

			return this.moveTo(armCoordsPoint);
		}

		/**
		 * First applies x- and y-coordinates and second the z-coordinate of the arm
		 * to the desired point.
		 * 
		 * @param p
		 */
		private void goToFrontally(Point3D p) {
			this.goToFrontally(p, 0);
		}

		private boolean goToFrontally(Point3D p, double distance) {
			try {
				double[][] kinectCoordsSource = {
						{ p.getX(LengthUnit.MILLIMETER) },
						{ p.getY(LengthUnit.MILLIMETER) },
						{ p.getZ(LengthUnit.MILLIMETER) }, { 1.0 } };
				Matrix kinectCoords = new Matrix(kinectCoordsSource);
				Matrix armCoords = this.cat.transform(kinectCoords);

				double armX = armCoords.get(0, 0) / armCoords.get(3, 0);
				double armY = armCoords.get(1, 0) / armCoords.get(3, 0);
				double armZ = armCoords.get(2, 0) / armCoords.get(3, 0);

				Point3D armCoordsPoint = new Point3D(armX, armY, armZ,
						LengthUnit.MILLIMETER);

				System.out.println("Going frontally to:");
				System.out.println("  Kinect: " + kinectCoords.get(0, 0) + ", "
						+ kinectCoords.get(1, 0) + ", " + kinectCoords.get(2, 0));
				System.out.println("     Arm: " + armX + ", " + armY + ", " + armZ);

				if (!checkGraspable(p)) {
					System.out.println("ABORT!! Point not graspable");
					return false;
				}

				Point3D xyPoint = new Point3D(armX, armY, armZ,
						LengthUnit.MILLIMETER);
				xyPoint.setZ(
						poseActuator.getPosition().getZ(LengthUnit.MILLIMETER),
						LengthUnit.MILLIMETER);
				boolean success = moveTo(xyPoint);
				armCoordsPoint.setZ(armCoordsPoint.getZ(LengthUnit.MILLIMETER)
						- distance, LengthUnit.MILLIMETER);
				return (success && moveTo(armCoordsPoint));
			} catch (Exception e) {
				Logger.getRootLogger().error("Error while going frontally.");
				return false;
			}
		}

		public boolean checkGraspable(Point3D pos) {
			double[][] kinectCoordsSource = { { pos.getX(LengthUnit.MILLIMETER) },
					{ pos.getY(LengthUnit.MILLIMETER) },
					{ pos.getZ(LengthUnit.MILLIMETER) }, { 1.0 } };
			Matrix kinectCoords = new Matrix(kinectCoordsSource);
			Matrix armCoords = this.cat.transform(kinectCoords);
			System.out.println("grasping point in armcoordinates: x= "
					+ armCoords.get(0, 0) + " y= " + armCoords.get(1, 0) + "z="
					+ armCoords.get(2, 0));

			logger.info("grasping point in armcoordinates: x= "
					+ armCoords.get(0, 0) + " y= " + armCoords.get(1, 0) + "z="
					+ armCoords.get(2, 0));
			if ((armCoords.get(0, 0) < GRASP_MIN_X)
					|| (armCoords.get(0, 0) > GRASP_MAX_X)
					|| (armCoords.get(2, 0) > GRASP_MAX_Z)
					|| (armCoords.get(1, 0) < GRASP_MIN_Y)
					|| (armCoords.get(1, 0) > GRASP_MAX_Y)) {
				return false;
			} else {
				return true;
			}

		}

		/**
		 * Moves the arm <b>distance</b> millimeters in front of the desired point
		 * <b>p</b> and tries to grasp it.
		 * 
		 * @param p
		 *            The position of the object.
		 * @param distance
		 *            Distance in millimeters to keep to the point <b>p</b>.
		 */
		public boolean grasp(Point3D p) {
			try {
				this.poseActuator.openGripper();
				this.waitAfterMovement();
				boolean success = this.goTo(p, GRASP_DEFAULT_DIST);
				if (success) {
					int stepCount = 0;
					PoseData targetPose = new PoseData(
							p.getX(LengthUnit.MILLIMETER),
							p.getY(LengthUnit.MILLIMETER),
							p.getZ(LengthUnit.MILLIMETER), LengthUnit.MILLIMETER);
					// this.poseActuator.executeStrategy(targetPose);
					this.poseActuator.executeGraspMovement();
					/*
					 * while (this.poseActuator.getGipperSensorData()
					 * .getInfraredMiddle() > GRASP_MAX_APPROACH_DIST &&
					 * this.poseActuator.getPosition().getZ() + GRASP_APPROACH_Z *
					 * GRASP_APPROACH_STEP < GRASP_MAX_APPROACH_Z && stepCount++ <
					 * GRASP_MAX_APPROACH_STEP_COUNT) { double left, right; if
					 * (p.getY(LengthUnit.MILLIMETER) >= 200) { // up left =
					 * this.poseActuator.getGipperSensorData()
					 * .getInfraredLeftInsideNear(); right =
					 * this.poseActuator.getGipperSensorData()
					 * .getInfraredRightInsideNear(); } else { // down left =
					 * this.poseActuator.getGipperSensorData()
					 * .getInfraredRightInsideNear(); right =
					 * this.poseActuator.getGipperSensorData()
					 * .getInfraredLeftInsideNear(); } double bottom =
					 * Math.min(this.poseActuator.getGipperSensorData()
					 * .getInfraredRightFront(), this.poseActuator
					 * .getGipperSensorData() .getInfraredLeftFront()); double dy;
					 * if (bottom > 160) { // enough space to go down! dy = -10; }
					 * else { dy = 0; } double dx = (left + right) / 2 - left;
					 * 
					 * /* this.adjustCurrentPosition(GRASP_APPROACH_X *
					 * GRASP_APPROACH_STEP, 0, GRASP_APPROACH_Z *
					 * GRASP_APPROACH_STEP);\/
					 * 
					 * this.adjustCurrentPosition(0, dy, GRASP_APPROACH_STEP); }
					 */
					/*
					 * if (this.poseActuator.getPosition().getZ() + GRASP_APPROACH_Z
					 * GRASP_APPROACH_STEP < GRASP_MAX_APPROACH_Z || stepCount++ <
					 * GRASP_MAX_APPROACH_STEP_COUNT) { success = false; }
					 */
					success = this.closeGripperByForce();
					// this.waitAfterMovement();
				}
				return success;
			} catch (IOException e) {
				Logger.getRootLogger().error("Error while grasping.");
				e.printStackTrace();
				return false;
			}
		}

		/**
		 * Applies the x- and y-coordinates of the arm to the desired point and then
		 * grasps the object frontally.
		 * 
		 * @param p
		 *            The position of the object.
		 * @return
		 */
		public boolean graspFrontally(Point3D p) {
			boolean ret = this.goToFrontally(p, GRASP_DEFAULT_DIST);
			return (ret && this.grasp(p));
		}

		/**
		 * Effectively moves the arm to the position given by a 4x1 vector of x, y,
		 * z coordinates. Depending on the value of y the arm tries to grasp up or
		 * down respectively.
		 * 
		 * @param armCoords
		 */
		public boolean moveTo(Point3D armCoords) {
			LengthUnit unit = LengthUnit.MILLIMETER;
			Logger.getRootLogger().info(
					"MoveTo called: \n" + "\t x: " + armCoords.getX(unit) + "\n"
							+ "\t y: " + armCoords.getY(unit) + "\n" + "\t z: "
							+ armCoords.getZ(unit) + "\n");

			double phi, psi, theta;
			try {
				// 300 is a magic number... decides if upper or lower grasp
				if (armCoords.getY(unit) < 300) {
					// Greifen unten
					// phi = 3.087451219558716;
					phi = Math.PI;
					/*
					 * - Math.atan(armCoords.get(0, 0) / armCoords.get(2, 0));
					 */
					psi = -Math.PI / 2; // -1.567844986915588;
					theta = Math.PI / 2; // 1.500036001205444;
					if (this.lastHeight == UP) {
						// Twist_down ausführen
						this.poseActuator.setMovements("twist_down");
						waitAfterMovement();
						// this.carryAngles = this.poseActuator.listJoints();
						this.carry = this.getPosition();
					}
					this.lastHeight = DOWN;
				} else {
					// Greifen oben
					phi = Math.PI;
					/*
					 * + Math.atan(armCoords.get(0, 0) / armCoords.get(2, 0));
					 */
					psi = Math.PI / 2;
					theta = Math.PI / 2;
					if (this.lastHeight == DOWN) {
						// twist_up ausführen
						this.poseActuator.setMovements("twist_up");
						this.waitAfterMovement();
						// this.carryAngles = this.poseActuator.listJoints();
						this.carry = this.getPosition();
					}
					this.lastHeight = UP;
				}
				// Calculate direction for grasping
				double normFac = 1 / (armCoords.getX(unit) + armCoords.getZ(unit));
				this.GRASP_APPROACH_X = normFac * armCoords.getX(unit);
				this.GRASP_APPROACH_Z = normFac * armCoords.getZ(unit);
				KatanaPoseData newPoint = new KatanaPoseData(armCoords.getX(unit),
						armCoords.getY(unit), armCoords.getZ(unit), phi, psi,
						theta, AngleUnit.RADIAN, unit);
				boolean success = this.poseActuator.goTo(newPoint, true);
				success = true;
				waitAfterMovement();
				return success;
			} catch (IOException e) {
				Logger.getRootLogger().error("Error moving arm!");
				e.printStackTrace();
				return false;
			}
		}

		public void setCarry() {
			try {
				this.carry = this.poseActuator.getPosition();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		/**
		 * Helper function to test the infrared sensors.
		 */
		public void readSensors() {
			try {
				Logger.getRootLogger().setLevel(Level.OFF);
				// this.poseActuator.setMovements("grasp_up_init");
				this.waitAfterMovement();
				while (true) {
					/*
					 * if
					 * (this.poseActuator.getGipperSensorData().getInfraredMiddle()
					 * < GRASP_MAX_APPROACH_DIST) { System.out.println("middle"); }
					 * else if (this.poseActuator.getGipperSensorData()
					 * .getInfraredLeftFront() < GRASP_MAX_APPROACH_DIST) {
					 * System.out.println("leftFront"); } else if
					 * (this.poseActuator.getGipperSensorData()
					 * .getInfraredRightFront() < GRASP_MAX_APPROACH_DIST) {
					 * System.out.println("rightFront"); } else if
					 * (this.poseActuator.getGipperSensorData()
					 * .getInfraredLeftInsideFar() < GRASP_MAX_APPROACH_DIST) {
					 * System.out.println("leftInsideFar"); } else if
					 * (this.poseActuator.getGipperSensorData()
					 * .getInfraredRightInsideFar() < GRASP_MAX_APPROACH_DIST) {
					 * System.out.println("rightInsideFar"); } else if
					 * (this.poseActuator.getGipperSensorData()
					 * .getInfraredLeftInsideNear() < GRASP_MAX_APPROACH_DIST) {
					 * System.out.println("leftInsideNear"); } else if
					 * (this.poseActuator.getGipperSensorData()
					 * .getInfraredRightInsideNear() < GRASP_MAX_APPROACH_DIST) {
					 * System.out.println("rightInsideNear"); } else if
					 * (this.poseActuator.getGipperSensorData()
					 * .getInfraredLeftOutside() < GRASP_MAX_APPROACH_DIST) {
					 * System.out.println("leftOutside"); } else if
					 * (this.poseActuator.getGipperSensorData()
					 * .getInfraredRightOutside() < GRASP_MAX_APPROACH_DIST) {
					 * System.out.println("rightOutside"); } else {
					 * System.out.println(""); }
					 */
					String out = "\n\n\n\n\n\n\n\n\n";
					out += "Middle = "
							+ this.poseActuator.getGipperSensorData()
									.getInfraredMiddle();
					out += "\nLeftFront = "
							+ this.poseActuator.getGipperSensorData()
									.getInfraredLeftFront();
					out += "\nLeftInsideFar = "
							+ this.poseActuator.getGipperSensorData()
									.getInfraredLeftInsideFar();
					out += "\nLeftInsideNear = "
							+ this.poseActuator.getGipperSensorData()
									.getInfraredLeftInsideNear();
					out += "\nLeftOutside = "
							+ this.poseActuator.getGipperSensorData()
									.getInfraredLeftOutside();
					out += "\nRightFront = "
							+ this.poseActuator.getGipperSensorData()
									.getInfraredRightFront();
					out += "\nRightInsideFar  = "
							+ this.poseActuator.getGipperSensorData()
									.getInfraredRightInsideFar();
					out += "\nRightInsideNear = "
							+ this.poseActuator.getGipperSensorData()
									.getInfraredRightInsideNear();
					out += "\nRightOutside = "
							+ this.poseActuator.getGipperSensorData()
									.getInfraredRightOutside();
					out += "\nForceLeftInsideFar = "
							+ this.poseActuator.getGipperSensorData()
									.getForceLeftInsideFar();
					out += "\nForceLeftInsideNear = "
							+ this.poseActuator.getGipperSensorData()
									.getForceLeftInsideNear();
					out += "\nForceRightInsideFar = "
							+ this.poseActuator.getGipperSensorData()
									.getForceRightInsideFar();
					out += "\nForceRightInsideNear = "
							+ this.poseActuator.getGipperSensorData()
									.getForceRightInsideNear();
					System.out.println(out);
				}

			} catch (IOException e) {
				System.err.println("An error occured while getting infrared data:");
				e.printStackTrace();
			}
		}

		public static final long SLEEP = 50;
		public static final double threshold = 100;

		private boolean contentExists = false;

		public synchronized boolean waitForGripperContent(long timeoutMillis)
				throws InterruptedException {

			contentExists = false;
			Thread analyseGripperThread = new Thread() {

				@Override
				public void run() {

					double val;

					try {
						while (!isInterrupted()) {
							val = Math
									.min(poseActuator.getGipperSensorData()
											.getInfraredMiddle(),
											Math.min(
													Math.min(
															poseActuator
																	.getGipperSensorData()
																	.getInfraredLeftInsideFar(),
															poseActuator
																	.getGipperSensorData()
																	.getInfraredLeftInsideNear()),
													Math.min(
															poseActuator
																	.getGipperSensorData()
																	.getInfraredRightInsideFar(),
															poseActuator
																	.getGipperSensorData()
																	.getInfraredRightInsideNear())));

							// logger.debug("IR Value is: " +val);
							if (val <= threshold) {
								// content inserted
								contentExists = true;
								break;
							}
							try {
								Thread.sleep(SLEEP);
							} catch (InterruptedException ex) {
								Thread.interrupted();
								break;
							}
						}
					} catch (IOException e) {
						logger.error(
								"An error occured while getting infrared data!", e);
					}
				}
			};

			analyseGripperThread.start();
			analyseGripperThread.join(timeoutMillis);
			if (contentExists) {
				return true;
			}

			analyseGripperThread.interrupt();
			analyseGripperThread.join();
			return false;

		}

		/**
		 * Set the transformation matrix
		 * 
		 * @param t
		 *            New transformation matrix.
		 */
		public void setTransformationMatrix(Matrix t) {
			this.cat.setTransformationMatrix(t);
		}

		/**
		 * Open gripper.
		 * 
		 * @throws IOException
		 */
		public boolean openGripper() {
			try {
				poseActuator.openGripper();
				this.waitAfterMovement();
				return true;
			} catch (IOException e) {
				e.printStackTrace();
				return false;
			}
		}

		/**
		 * Open gripper when touched from the outside.
		 * 
		 * @throws IOException
		 */
		public void openGripperWhenTouching(int waitSeconds) throws IOException {
			poseActuator.openGripperWhenTouching(waitSeconds);
			waitAfterMovement();
		}

		/**
		 * Set the transformation matrix
		 * 
		 * @param filename
		 *            Path to transformation matrix file.
		 */
		public void setTransformationMatrix(String filename) {
			this.cat.setTransformationMatrix(filename);
		}

		/**
		 * Since the control of the Katana arm is vulnerable to multiple commands
		 * sent in too short time slots, it is necessary to wait for a few seconds
		 * after each movement. 3 seconds worked just fine so far, but this depends
		 * on both, the way ToBI and the Katana arm feels.
		 */
		public void waitAfterMovement() {
			waitAfterMovement(3000);
		}

		/**
		 * Since the control of the Katana arm is vulnerable to multiple commands
		 * sent in too short time slots, it is necessary to wait for a few seconds
		 * after each movement. 3 seconds worked just fine so far, but this depends
		 * on both, the way ToBI and the Katana arm feels.
		 * 
		 * @param milli
		 *            Number of seconds to wait.
		 */
		public void waitAfterMovement(long time) {
			try {
				Thread.sleep(time);
			} catch (InterruptedException ex) {
				logger.warn("Caught interrupt during movement wait!", ex);
			}
		}

		public void feature(String f) {
			ArrayList<String> availableFeatures = new ArrayList<String>();
			availableFeatures.add("wave_left");
			availableFeatures.add("wave_half_left");
			availableFeatures.add("wave_middle");
			availableFeatures.add("wave_half_right");
			availableFeatures.add("wave_right");
			availableFeatures.add("wave_out");
			availableFeatures.add("sword_hit");
			availableFeatures.add("up_init");

			if (availableFeatures.contains(f)) {
				try {
					this.poseActuator.setMovements(f);
					this.waitAfterMovement(1000);
				} catch (Exception e) {
					logger.debug(e);
				}
			}
		}

		public void goOutUp() {
			try {
				this.poseActuator.setMovements("grasp_up_humble");
				this.waitAfterMovement();
				lastHeight = UP;
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
